# Library for utilizing the Puppet REST API for certificates

require 'net/https'

module CertAPI

  def self.resource_type(pluralize = false)
    val = "certificate_status"
    val += "es" if pluralize
    val
  end

  def self.http
    return @http unless @http.nil?

    begin
      key_content = IO.read(@cert_key_path)
    rescue => e
      raise "Could not read file #{@cert_key_path}: #{e}"
    end

    begin
      cert_content = IO.read(@cert_path)
    rescue => e
      raise "Could not read file #{@cert_path}: #{e}"
    end

    @http = Net::HTTP.new(@ca_location, @ca_port)

    @http.ca_file = @ca_file_path
    @http.cert = OpenSSL::X509::Certificate.new(cert_content)
    @http.key = OpenSSL::PKey::RSA.new(key_content)

    @http.use_ssl = true
    @http.verify_mode = OpenSSL::SSL::VERIFY_PEER
    @http
  end

  def self.config(options = {})
    @ca_file_path = options[:ca_file_path] || ''
    @cert_path = options[:cert_path] || ''
    @cert_key_path = options[:cert_key_path] || ''
    @ca_location = options[:ca_location] || 'localhost'
    @ca_port = options[:ca_port] || '8140'
    @environment = options[:environment] || 'production'
    @http = nil
  end

  def self.uri(name, pluralize = false)
    "/#{@environment}/#{resource_type(pluralize)}/#{name}"
  end

  # We are forcing pson here due to issues with the certificate metadata
  # returned by the Puppet REST API. The REST API returns inconsistent
  # metadata between formats (ticket #15730), but pson was patched to
  # include the full set of metadata (ticket #15731).

  def self.data(certname='all')
    should_pluralize = certname == 'all'
    api_request = Net::HTTP::Get.new(uri(certname, should_pluralize), initheader = {'Accept' => 'pson'})
    api_response = http.request(api_request)

    if api_response.code == '500'
      raise "Error retrieving certificates from puppet API (it returned a 500 error)"
    end

    begin
      JSON.parse(api_response.body)
    rescue JSON::ParserError
      # The response is not always in JSON/PSON
      api_response.body
    end
  end

  # Get metadata for all certificates, sorted by state and name
  def self.certificate_statuses
    data.sort_by { |certificate| [certificate['state'],certificate['name']] }
  end

  # Get metadata for certificate requests
  def self.csrs
    csrs_list = []
    certificate_statuses.each do |c|
      if c['state'] == 'requested'
       csrs_list << c
      end
    end
    csrs_list
  end

  # Get a list of certnames for all certificates
  def self.certnames
    names = []
    certificate_statuses.each do |c|
      names << c['name']
    end
    names
  end

  # Get a list of certnames for just the certificate requests
  def self.csr_names
    names = []
    csrs.each do |c|
      names << c['name']
    end
    names
  end

  # Get metadata for specified certname
  def self.cert_info(name)
    data(name) || []
  end

  # Sign a certificate
  def self.add(name)
    api_request = Net::HTTP::Put.new(uri(name), initheader = {'Content-Type' => 'text/pson'})
    api_request.body = '{"desired_state":"signed"}'
    http.request(api_request)
  end

  # Reject a certificate request
  def self.remove(name)
    api_request = Net::HTTP::Delete.new(uri(name))
    http.request(api_request)
  end

  config
end
