require 'ar-extensions'
require 'ar-extensions/insert_select/mysql'

class Baseline < ActiveRecord::Base
  unloadable

  belongs_to :node
  has_many :resources,
    :class_name => "BaselineResource",
    :dependent => :destroy,
    :order => 'LOWER(resource_type) ASC, LOWER(title) ASC'

  accepts_nested_attributes_for :resources

  def self.create_from_report!(report)
    raise IncorrectReportKind.new("expected 'inspect', got '#{report.kind}'") unless report.kind == "inspect"

    baseline = nil
    ActiveRecord::Base.transaction do
      baseline = Baseline.create!

      BaselineResource.insert_select(:from       => :resource_statuses,
                                     :conditions => {
                                       :report_id => report.id,
                                       :failed    => false
                                     },
                                     :select     => ['resource_type, title, ?', baseline.id],
                                     :into       => [:resource_type, :title, :baseline_id])

      # WARNING: DANGER!  This is deadlock-prone if you do it concurrently in
      # the database.  It winds up failing in horrible ways, and needs to be
      # serialized somehow.  At the moment we only invoke it in one thread, so
      # we can use this fast code, but if you change that...
      join_sql = <<-JOIN_SQL
        INNER JOIN resource_statuses
          ON  resource_statuses.report_id = #{report.id}
          AND resource_statuses.id        = resource_events.resource_status_id
        INNER JOIN baseline_resources
          ON  baseline_resources.baseline_id  = #{baseline.id}
          AND resource_statuses.resource_type = baseline_resources.resource_type
          AND resource_statuses.title         = baseline_resources.title
      JOIN_SQL

      BaselineProperty.insert_select(:from   => ResourceEvent,
                                     :select => '`baseline_resources`.`id`, `property`, `previous_value`',
                                     :into   => [:baseline_resource_id, :name, :value],
                                     :joins  => join_sql)
    end

    baseline
  end

  def deep_clone
    new_baseline = Baseline.create!
    BaselineResource.insert_select(:from       => BaselineResource,
                                   :conditions => {:baseline_id => self.id},
                                   :select     => ['resource_type, title, ?', new_baseline.id ],
                                   :into       => [:resource_type, :title, :baseline_id])

    # WARNING: DANGER!  This looks to be as deadlock-prone as the code in
    # create_from_report!.  That hasn't shown up in the real world, but if we
    # are unlucky...
    #
    # On the plus side, this, like the other code, runs serialized in the
    # daily maintenance task, so we can use this nice, fast version here.
    #
    # Again, if you plan on changing that, watch out.  Dragons live here.
    join_sql = <<-JOIN_SQL
      INNER JOIN baseline_resources old_resources
        ON  old_resources.baseline_id = #{self.id}
        AND old_resources.id          = baseline_properties.baseline_resource_id
      INNER JOIN baseline_resources new_resources
        ON  new_resources.baseline_id   = #{new_baseline.id}
        AND old_resources.resource_type = new_resources.resource_type
        AND old_resources.title         = new_resources.title
    JOIN_SQL

    BaselineProperty.insert_select(:from   => BaselineProperty,
                                   :select => '`new_resources`.`id`, `name`, `value`',
                                   :into   => [:baseline_resource_id, :name, :value],
                                   :joins  => join_sql)

    new_baseline
  end

  def self.find_current_by_node_id(node_id)
    self.find_by_node_id(node_id, :order => "date DESC")
  end
end
